from django.conf.urls import patterns, url
from django.views.generic import TemplateView

from .feeds import ArticleAtomFeed, ArticleRssFeed
from .views import ArticleView, ArticleList


urlpatterns = patterns('',
    url(r'^$', ArticleList.as_view(), name='list_articles'),
    url(r'^feed/$', TemplateView.as_view(template_name='chalk/feed_list.html'),
        name='list_feeds'),
    url(r'^feed/rss/$', ArticleRssFeed(), name='rss_feed'),
    url(r'^feed/atom/$', ArticleAtomFeed(), name='atom_feed'),
    url(r'^(?P<slug>[\w-]+)/$', ArticleView.as_view(), name='view_article'),
)



from django.views.generic import DetailView, ListView

from .models import Article


class ArticleList(ListView):
    """
    List all articles by publication date descending.

    The default template extends the 'container' block of 'base.html'
    and is meant mostly as an example. Override 'chalk/article_list.html'
    to use your own template.

    Articles are provided to the template context in the 'article_list'
    variable.

    """
    model = Article

    def get_queryset(self):
        """Non-staff users only see published articles."""
        qs = self.model.objects.all()
        if not self.request.user.is_staff:
            qs = qs.filter(published=True)
        return qs


class ArticleView(DetailView):
    """
    Show a detailed view of a single article.

    The default template extends the 'container' block of 'base.html'
    and is meant mostly as an example. Override 'chalk/article_detail.html'
    to use your own template.

    You can access the article being viewed through the 'article' variable
    in the template context.

    """
    model = Article

    def get_queryset(self):
        """Non-staff users only see published articles."""
        qs = self.model.objects.all()
        if not self.request.user.is_staff:
            qs = qs.filter(published=True)
        return qs



from datetime import datetime, time

from django.contrib.syndication.views import Feed
from django.core.urlresolvers import reverse
from django.utils.feedgenerator import Atom1Feed

from .models import Article
from .settings import FEED_TITLE, FEED_DESCRIPTION


class ArticleRssFeed(Feed):

    def title(self):
        return FEED_TITLE

    def description(self):
        return FEED_DESCRIPTION

    def link(self):
        return reverse('list_articles')

    def items(self):
        return Article.objects.filter(published=True)

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.excerpt_html

    def item_pubdate(self, item):
        #TODO: looks like publication_date might be better as a DateTimeField
        return datetime.combine(item.publication_date, time())

    def item_author_name(self, item):
        return item.author.get_full_name()

    def item_author_email(self, item):
        return item.author.email


class ArticleAtomFeed(ArticleRssFeed):
    feed_type = Atom1Feed
    subtitle = ArticleRssFeed.description



from django.contrib import admin

from .models import Article


class ArticleAdmin(admin.ModelAdmin):
    list_display = ('title', 'published', 'publication_date',)
    prepopulated_fields = {'slug': ('title',)}
    fieldsets = (
        (None, {
            'fields': ('author', 'title', 'content', 'excerpt', 'published',
                       'publication_date')
        }),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': ('protect_html', 'content_html', 'excerpt_html', 'slug',
                       'meta_description')
        }),
    )


admin.site.register(Article, ArticleAdmin)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.contrib.auth import get_user_model
from django.db import models


User = get_user_model()

user_orm_label = '%s.%s' % (User._meta.app_label, User._meta.object_name)
user_model_label = '%s.%s' % (User._meta.app_label, User._meta.module_name)

class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Article.meta_description'
        db.add_column(u'chalk_article', 'meta_description',
                      self.gf('django.db.models.fields.TextField')(default='', blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Article.meta_description'
        db.delete_column(u'chalk_article', 'meta_description')


    models = {
        user_model_label: {
            'Meta': {'object_name': User.__name__, 'db_table': "'%s'" % User._meta.db_table},
            User._meta.pk.attname: ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
        },
        u'chalk.article': {
            'Meta': {'ordering': "['-publication_date']", 'object_name': 'Article'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['%s']" % user_orm_label}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'content_html': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'excerpt': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'excerpt_html': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'meta_description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'protect_html': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'publication_date': ('django.db.models.fields.DateField', [], {'default': 'datetime.datetime(2013, 8, 25, 0, 0)'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '250'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        },
    }

    complete_apps = ['chalk']



from django.test import TestCase
from django.core.urlresolvers import reverse

from .models import Article


class ArticleViewTests(TestCase):
    """Test article views."""
    fixtures = ['chalk_test_data.json']

    def test_list_view(self):
        """Should only return published articles for non-staff users."""
        published_articles = Article.objects.filter(published=True)
        response = self.client.get(reverse('list_articles'))
        returned_articles = response.context['article_list']
        self.assertItemsEqual(returned_articles, published_articles)

    def test_detail_view(self):
        """Check that article detail view returns correct article."""
        article = Article.objects.filter(published=True)[0]
        response = self.client.get(reverse('view_article',
                                           kwargs={'slug': article.slug}))
        returned_article = response.context['article']
        self.assertEqual(article, returned_article)

    def test_detail_view_unpublished(self):
        """Unpublished articles should return 404 for non-staff users."""
        article = Article.objects.filter(published=False)[0]
        response = self.client.get(reverse('view_article',
                                           kwargs={'slug': article.slug}))
        self.assertEqual(response.status_code, 404)



class ArticleModelTests(TestCase):
    """Test the Article model."""
    fixtures = ['chalk_test_data.json']

    def test_save(self):
        """Saving should populate HTML fields by default."""
        article = Article.objects.get(title="Test Title") #From fixture
        article.content_html = "content to be overwritten"
        article.excerpt_html = "excerpt to be overwritten"
        article.save()
        self.assertEqual(article.content_html, "<p>Some content</p>\n")
        self.assertEqual(article.excerpt_html , "<p>An excerpt</p>\n")

    def test_protect_html(self):
        """Article.protect_html should prevent HTML overwrites."""
        article = Article.objects.get(title="Test Title") #From fixture
        article.protect_html = True
        article.content_html = "Protected Content"
        article.excerpt_html = "Protected Excerpt"
        article.save()
        self.assertEqual(article.content_html, "Protected Content")
        self.assertEqual(article.excerpt_html , "Protected Excerpt")

    def test_meta_description_default(self):
        """
        Article.get_meta_description should be equivalent to Article.excerpt if
        Article.meta_description is empty.

        """
        article = Article.objects.get(title="Test Title") #From fixture
        article.excerpt = "A blurb about nothing."
        article.meta_description = " "
        article.save()
        self.assertEqual(article.get_meta_description(), "A blurb about nothing.")

    def test_meta_description_custom(self):
        """
        Article.get_meta_description should return Article.meta_description if
        it isn't empty.

        """
        article = Article.objects.get(title="Test Title") #From fixture
        article.meta_description = "A custom description"
        article.save()
        self.assertEqual(article.get_meta_description(), "A custom description")

    def test_generate_html(self):
        """Articles should generate correct HTML from reST input."""
        excerpt = """Here's a bit of `reStructuredText`_ for *testing* purposes.

.. _reStructuredText: http://docutils.sourceforge.net/rst.html
"""

        expected_html = ("""<div class="section" id="glorious-content">
<h2>Glorious Content</h2>
<p>What about an internal reference to a <a class="reference internal" href="#section">Section</a>?</p>
<div class="section" id="section">
<h3>Section</h3>
<p>Great.</p>
</div>
</div>
""",
"""<p>Here's a bit of <a class="reference external" href="http://docutils.sourceforge.net/rst.html">reStructuredText</a> for <em>testing</em> purposes.</p>
""")

        content = """Glorious Content
----------------

What about an internal reference to a `Section`_?

Section
=======

Great."""

        article = Article.objects.get(title="Test Title") #From fixture
        article.excerpt = excerpt
        article.content = content

        self.assertEqual(article.generate_html(), expected_html)
